@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     pygments.lexers.archetype
     ~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -14,13 +13,13 @@
     Contributed by Thomas Beale <https://github.com/wolandscat>,
     <https://bitbucket.org/thomas_beale>.
 
-    :copyright: Copyright 2006-2019 by the Pygments team, see AUTHORS.
+    :copyright: Copyright 2006-2022 by the Pygments team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from pygments.lexer import RegexLexer, include, bygroups, using, default
 from pygments.token import Text, Comment, Name, Literal, Number, String, \
-    Punctuation, Keyword, Operator, Generic
+    Punctuation, Keyword, Operator, Generic, Whitespace
 
 __all__ = ['OdinLexer', 'CadlLexer', 'AdlLexer']
 
@@ -35,13 +34,14 @@ class AtomsLexer(RegexLexer):
     tokens = {
         # ----- pseudo-states for inclusion -----
         'whitespace': [
-            (r'\n', Text),
-            (r'\s+', Text),
-            (r'[ \t]*--.*$', Comment),
+            (r'\n', Whitespace),
+            (r'\s+', Whitespace),
+            (r'([ \t]*)(--.*)$', bygroups(Whitespace, Comment)),
         ],
         'archetype_id': [
-            (r'[ \t]*([a-zA-Z]\w+(\.[a-zA-Z]\w+)*::)?[a-zA-Z]\w+(-[a-zA-Z]\w+){2}'
-             r'\.\w+[\w-]*\.v\d+(\.\d+){,2}((-[a-z]+)(\.\d+)?)?', Name.Decorator),
+            (r'([ \t]*)(([a-zA-Z]\w+(\.[a-zA-Z]\w+)*::)?[a-zA-Z]\w+(-[a-zA-Z]\w+){2}'
+             r'\.\w+[\w-]*\.v\d+(\.\d+){,2}((-[a-z]+)(\.\d+)?)?)',
+             bygroups(Whitespace, Name.Decorator)),
         ],
         'date_constraints': [
             # ISO 8601-based date/time constraints
@@ -58,7 +58,7 @@ class AtomsLexer(RegexLexer):
             (r'P((\d*(\.\d+)?[YyMmWwDd]){1,3}(T(\d*(\.\d+)?[HhMmSs]){,3})?|'
              r'T(\d*(\.\d+)?[HhMmSs]){,3})', Literal.Date),
             (r'[+-]?(\d+\.\d*|\.\d+|\d+)[eE][+-]?\d+', Number.Float),
-            (r'[+-]?(\d+)*\.\d+%?', Number.Float),
+            (r'[+-]?\d*\.\d+%?', Number.Float),
             (r'0x[0-9a-fA-F]+', Number.Hex),
             (r'[+-]?\d+%?', Number.Integer),
         ],
@@ -108,7 +108,7 @@ class AtomsLexer(RegexLexer):
             (r'[<>=] *', Punctuation),
             # handle +/-
             (r'\+/-', Punctuation),
-            (r'\s+', Text),
+            (r'\s+', Whitespace),
         ],
         'any_code': [
             include('archetype_id'),
@@ -121,7 +121,7 @@ class AtomsLexer(RegexLexer):
             (r'\|', Punctuation, 'code_rubric'),
             (r'\]', Punctuation, '#pop'),
             # handle use_archetype statement
-            (r'\s*,\s*', Punctuation),
+            (r'(\s*)(,)(\s*)', bygroups(Whitespace, Punctuation, Whitespace)),
         ],
         'code_rubric': [
             (r'\|', Punctuation, '#pop'),
@@ -154,8 +154,8 @@ class OdinLexer(AtomsLexer):
             (r'[a-z_]\w*', Name.Class),
             (r'/', Punctuation),
             (r'\[', Punctuation, 'key'),
-            (r'\s*,\s*', Punctuation, '#pop'),
-            (r'\s+', Text, '#pop'),
+            (r'(\s*)(,)(\s*)', bygroups(Whitespace, Punctuation, Whitespace), '#pop'),
+            (r'\s+', Whitespace, '#pop'),
         ],
         'key': [
             include('values'),
@@ -212,16 +212,16 @@ class CadlLexer(AtomsLexer):
             (r'(not)\W', Operator),
             (r'(matches|is_in)\W', Operator),
             # is_in / not is_in char
-            (u'(\u2208|\u2209)', Operator),
+            ('(\u2208|\u2209)', Operator),
             # there_exists / not there_exists / for_all / and / or
-            (u'(\u2203|\u2204|\u2200|\u2227|\u2228|\u22BB|\223C)',
+            ('(\u2203|\u2204|\u2200|\u2227|\u2228|\u22BB|\223C)',
              Operator),
             # regex in slot or as string constraint
-            (r'(\{)(\s*/[^}]+/\s*)(\})',
-             bygroups(Punctuation, String.Regex, Punctuation)),
+            (r'(\{)(\s*)(/[^}]+/)(\s*)(\})',
+             bygroups(Punctuation, Whitespace, String.Regex, Whitespace, Punctuation)),
             # regex in slot or as string constraint
-            (r'(\{)(\s*\^[^}]+\^\s*)(\})',
-             bygroups(Punctuation, String.Regex, Punctuation)),
+            (r'(\{)(\s*)(\^[^}]+\^)(\s*)(\})',
+             bygroups(Punctuation, Whitespace, String.Regex, Whitespace, Punctuation)),
             (r'/', Punctuation, 'path'),
             # for cardinality etc
             (r'(\{)((?:\d+\.\.)?(?:\d+|\*))'
@@ -264,16 +264,17 @@ class AdlLexer(AtomsLexer):
     tokens = {
         'whitespace': [
             # blank line ends
-            (r'\s*\n', Text),
+            (r'\s*\n', Whitespace),
             # comment-only line
-            (r'^[ \t]*--.*$', Comment),
+            (r'^([ \t]*)(--.*)$', bygroups(Whitespace, Comment)),
         ],
         'odin_section': [
             # repeating the following two rules from the root state enable multi-line
             # strings that start in the first column to be dealt with
             (r'^(language|description|ontology|terminology|annotations|'
-             r'component_terminologies|revision_history)[ \t]*\n', Generic.Heading),
-            (r'^(definition)[ \t]*\n', Generic.Heading, 'cadl_section'),
+             r'component_terminologies|revision_history)([ \t]*\n)',
+             bygroups(Generic.Heading, Whitespace)),
+            (r'^(definition)([ \t]*\n)', bygroups(Generic.Heading, Whitespace), 'cadl_section'),
             (r'^([ \t]*|[ \t]+.*)\n', using(OdinLexer)),
             (r'^([^"]*")(>[ \t]*\n)', bygroups(String, Punctuation)),
             # template overlay delimiter
@@ -300,7 +301,7 @@ class AdlLexer(AtomsLexer):
             (r'\w+', Name.Class),
             (r'"', String, 'string'),
             (r'=', Operator),
-            (r'[ \t]+', Text),
+            (r'[ \t]+', Whitespace),
             default('#pop'),
         ],
         'root': [
@@ -312,7 +313,7 @@ class AdlLexer(AtomsLexer):
             (r'^(definition)[ \t]*\n', Generic.Heading, 'cadl_section'),
             (r'^(rules)[ \t]*\n', Generic.Heading, 'rules_section'),
             include('archetype_id'),
-            (r'[ \t]*\(', Punctuation, 'metadata'),
+            (r'([ \t]*)(\()', bygroups(Whitespace, Punctuation), 'metadata'),
             include('whitespace'),
         ],
     }
