@@ -4,7 +4,7 @@
 
     Lexers for computer graphics and plotting related languages.
 
-    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
+    :copyright: Copyright 2006-2022 by the Pygments team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -306,13 +306,10 @@ class PostScriptLexer(RegexLexer):
     """
     Lexer for PostScript files.
 
-    The PostScript Language Reference published by Adobe at
-    <http://partners.adobe.com/public/developer/en/ps/PLRM.pdf>
-    is the authority for this.
-
     .. versionadded:: 1.4
     """
     name = 'PostScript'
+    url = 'https://en.wikipedia.org/wiki/PostScript'
     aliases = ['postscript', 'postscr']
     filenames = ['*.ps', '*.eps']
     mimetypes = ['application/postscript']
@@ -402,11 +399,12 @@ class PostScriptLexer(RegexLexer):
 
 class AsymptoteLexer(RegexLexer):
     """
-    For `Asymptote <http://asymptote.sf.net/>`_ source code.
+    For Asymptote source code.
 
     .. versionadded:: 1.2
     """
     name = 'Asymptote'
+    url = 'http://asymptote.sf.net/'
     aliases = ['asymptote', 'asy']
     filenames = ['*.asy']
     mimetypes = ['text/x-asymptote']
@@ -529,12 +527,13 @@ def _shortened_many(*words):
 
 class GnuplotLexer(RegexLexer):
     """
-    For `Gnuplot <http://gnuplot.info/>`_ plotting scripts.
+    For Gnuplot plotting scripts.
 
     .. versionadded:: 0.11
     """
 
     name = 'Gnuplot'
+    url = 'http://gnuplot.info/'
     aliases = ['gnuplot']
     filenames = ['*.plot', '*.plt']
     mimetypes = ['text/x-gnuplot']
@@ -688,11 +687,12 @@ class GnuplotLexer(RegexLexer):
 
 class PovrayLexer(RegexLexer):
     """
-    For `Persistence of Vision Raytracer <http://www.povray.org/>`_ files.
+    For Persistence of Vision Raytracer files.
 
     .. versionadded:: 0.11
     """
     name = 'POVRay'
+    url = 'http://www.povray.org/'
     aliases = ['pov']
     filenames = ['*.pov', '*.inc']
     mimetypes = ['text/x-povray']
@@ -766,14 +766,12 @@ class PovrayLexer(RegexLexer):
                 'quadric', 'quartic', 'smooth_triangle', 'sor', 'sphere', 'superellipsoid',
                 'text', 'torus', 'triangle', 'union'), suffix=r'\b'),
              Name.Builtin),
-            # TODO: <=, etc
-            (r'[\[\](){}<>;,]', Punctuation),
-            (r'[-+*/=]', Operator),
             (r'\b(x|y|z|u|v)\b', Name.Builtin.Pseudo),
             (r'[a-zA-Z_]\w*', Name),
-            (r'[0-9]+\.[0-9]*', Number.Float),
-            (r'\.[0-9]+', Number.Float),
+            (r'[0-9]*\.[0-9]+', Number.Float),
             (r'[0-9]+', Number.Integer),
+            (r'[\[\](){}<>;,]', Punctuation),
+            (r'[-+*/=.|&]|<=|>=|!=', Operator),
             (r'"(\\\\|\\[^\\]|[^"\\])*"', String),
             (r'\s+', Whitespace),
         ]
